#include <stdio.h>
#include <libplayerc/playerc.h>

int main(int argc, const char **argv)
{
  playerc_client_t *client;
  playerc_position2d_t *position2d;
  playerc_laser_t *laser;
  playerc_sonar_t *sonar;

  int i;
  float x;

// Conecta ao servidor
  client = playerc_client_create(NULL, "localhost", 6665);
  if (playerc_client_connect(client) != 0)
    return -1;

// Conecta ao position (comando e odometria)
  position2d = playerc_position2d_create(client, 0);
  if (playerc_position2d_subscribe(position2d, PLAYERC_OPEN_MODE) != 0)  return -1;

// Conecta ao laser
  laser = playerc_laser_create(client, 0);
  if (playerc_laser_subscribe(laser, PLAYERC_OPEN_MODE))
     return -1;

// Conecta ao sonar
//  sonar = playerc_sonar_create(client, 0);
//  if (playerc_sonar_subscribe(sonar, PLAYERC_OPEN_MODE))
//    return -1;
 
// Configura parametros
  if (playerc_client_datamode (client, PLAYERC_DATAMODE_PULL) != 0)
  {
     fprintf(stderr, "error: %s\n", playerc_error_str());
     return -1;
  }

// Configura parametros
  if (playerc_client_set_replace_rule (client, -1, -1, PLAYER_MSGTYPE_DATA, -1, 1) != 0)
  {
     fprintf(stderr, "error: %s\n", playerc_error_str());
     return -1;
  }

// Ativa os motores do robo
   playerc_position2d_enable(position2d, 1);

//Comando para o robô se mover para frente a 0,2m/s e rotacionar a 0.4rad/s
   playerc_position2d_set_cmd_vel(position2d, 0.4, 0.0, 0.0, 1);

   playerc_client_read(client);
   playerc_client_read(client);
   playerc_client_read(client);
   playerc_client_read(client);
   playerc_client_read(client);

// Mostra a posiçao do robô
 while(1)
 {
   //Recebe dados do servidor
     playerc_client_read(client);
   //Mostra a leitura do laser 0 (direita), 180 (frente) e 360 (esquerda)
     printf("Laser (DCE): %.3f %.3f %.3f \n", laser->scan[360][0], 
             laser->scan[180][0], laser->scan[0][0]);

    // Speed up se não tiver obstáculos próximos
    if  ((laser->scan[180][0] > 1.0) && 
         (laser->scan[140][0] > 1.0) && 
         (laser->scan[220][0] > 1.0) &&
         (laser->scan[0][0]   > 0.7) &&
         (laser->scan[359][0] > 0.7))
         playerc_position2d_set_cmd_vel(position2d, 0.4, 0.0, 0.0, 1);

    // Slow down se tiver obstáculos próximos
    if  ((laser->scan[180][0] < 1.0) ||
         (laser->scan[140][0] < 1.0) || 
         (laser->scan[220][0] < 1.0) ||
         (laser->scan[0][0]   < 0.7) ||
         (laser->scan[359][0] < 0.7))
         playerc_position2d_set_cmd_vel(position2d, 0.1, 0.0, 0.0, 1);

    // Gira para a esquerda de tiver um obstáculo muito próximo em frente
    if ((laser->scan[180][0] <  0.7) || 
         (laser->scan[140][0] < 0.7) || 
         (laser->scan[220][0] < 0.7) ||
         (laser->scan[0][0]   < 0.7) ||  
         (laser->scan[359][0] < 0.7))
         playerc_position2d_set_cmd_vel(position2d, 0.0, 0.0, 0.4, 1);
  
 }
	
  playerc_position2d_set_cmd_vel(position2d, 0.0, 0.0, 0.0, 1);

  playerc_laser_unsubscribe(laser);
  playerc_laser_destroy(laser);
  playerc_position2d_unsubscribe(position2d);
  playerc_position2d_destroy(position2d);
  playerc_client_disconnect(client);
  playerc_client_destroy(client);

  return 0;
}
